package org.processmining.plugins.xpdl.idname;

import java.util.ArrayList;
import java.util.List;

import org.processmining.plugins.xpdl.Xpdl;
import org.processmining.plugins.xpdl.XpdlElement;
import org.processmining.plugins.xpdl.collections.XpdlPerformers;
import org.processmining.plugins.xpdl.graphics.collections.XpdlNodeGraphicsInfos;
import org.xmlpull.v1.XmlPullParser;

/**
 * @author hverbeek
 * 
 *         <xsd:element name="Lane"> <xsd:annotation>
 *         <xsd:documentation>BPMN</xsd:documentation> </xsd:annotation>
 *         <xsd:complexType> <xsd:sequence minOccurs="0"> <xsd:element
 *         ref="xpdl:Object" minOccurs="0"/> <xsd:element
 *         ref="xpdl:NodeGraphicsInfos" minOccurs="0"/> <xsd:element
 *         ref="xpdl:Performers" minOccurs="0"/> <xsd:element name="NestedLane"
 *         minOccurs="0" maxOccurs="unbounded"> <xsd:complexType> <xsd:attribute
 *         name="LaneId" type="xsd:NMTOKEN" use="required"/> </xsd:complexType>
 *         </xsd:element> <xsd:any namespace="##other" processContents="lax"
 *         minOccurs="0" maxOccurs="unbounded"/> </xsd:sequence> <xsd:attribute
 *         name="Id" type="xsd:NMTOKEN" use="required"/> <xsd:attribute
 *         name="Name" type="xsd:string" use="optional"/> <xsd:attribute
 *         name="ParentLane" type="xsd:NMTOKEN" use="optional"> <xsd:annotation>
 *         <xsd:documentation>Deprecated from BPMN1.0. </xsd:documentation>
 *         </xsd:annotation> </xsd:attribute> <xsd:attribute name="ParentPool"
 *         type="xsd:NMTOKEN" use="optional"> <xsd:annotation>
 *         <xsd:documentation>Deprecated from BPMN1.0. </xsd:documentation>
 *         </xsd:annotation> </xsd:attribute> <xsd:anyAttribute
 *         namespace="##other" processContents="lax"/> </xsd:complexType>
 *         </xsd:element>
 */
public class XpdlLane extends XpdlIdName {

	private class XpdlNestedLaneId extends XpdlElement {
		/*
		 * Attributes
		 */
		private String laneId;

		public XpdlNestedLaneId(String tag) {
			super(tag);

			laneId = null;
		}

		protected void importAttributes(XmlPullParser xpp, Xpdl xpdl) {
			super.importAttributes(xpp, xpdl);
			String value = xpp.getAttributeValue(null, "LaneId");
			if (value != null) {
				laneId = value;
			}
		}

		/**
		 * Exports all attributes.
		 */
		protected String exportAttributes() {
			String s = super.exportAttributes();
			if (laneId != null) {
				s += exportAttribute("LaneId", laneId);
			}
			return s;
		}

		protected void checkValidity(Xpdl xpdl) {
			super.checkValidity(xpdl);
			checkRequired(xpdl, "LaneId", laneId);
		}
	}

	/*
	 * Attributes
	 */
	private String parentLane;
	private String parentPool;

	/*
	 * Elements
	 */
	private XpdlObject object;
	private XpdlNodeGraphicsInfos nodeGraphicsInfos;
	private XpdlPerformers performers;
	private final List<XpdlNestedLaneId> nestedLaneList;

	public XpdlLane(String tag) {
		super(tag);

		parentLane = null;
		parentPool = null;

		object = null;
		nodeGraphicsInfos = null;
		performers = null;
		nestedLaneList = new ArrayList<XpdlNestedLaneId>();
	}

	protected boolean importElements(XmlPullParser xpp, Xpdl xpdl) {
		if (super.importElements(xpp, xpdl)) {
			/*
			 * Start tag corresponds to a known child element of an XPDL node.
			 */
			return true;
		}
		if (xpp.getName().equals("Object")) {
			object = new XpdlObject("Object");
			object.importElement(xpp, xpdl);
			return true;
		}
		if (xpp.getName().equals("NodeGraphicsInfos")) {
			nodeGraphicsInfos = new XpdlNodeGraphicsInfos("NodeGraphicsInfos");
			nodeGraphicsInfos.importElement(xpp, xpdl);
			return true;
		}
		if (xpp.getName().equals("Performers")) {
			performers = new XpdlPerformers("Performers");
			performers.importElement(xpp, xpdl);
			return true;
		}
		if (xpp.getName().equals("NestedLane")) {
			XpdlNestedLaneId nestedLane = new XpdlNestedLaneId("NestedLane");
			nestedLane.importElement(xpp, xpdl);
			nestedLaneList.add(nestedLane);
			return true;
		}
		/*
		 * Unknown tag
		 */
		return false;
	}

	protected String exportElements() {
		/*
		 * Export node child elements.
		 */
		String s = super.exportElements();
		if (object != null) {
			s += object.exportElement();
		}
		if (nodeGraphicsInfos != null) {
			s += nodeGraphicsInfos.exportElement();
		}
		if (performers != null) {
			s += performers.exportElement();
		}
		for (XpdlNestedLaneId nestedLane : nestedLaneList) {
			s += nestedLane.exportElement();
		}
		return s;
	}

	protected void importAttributes(XmlPullParser xpp, Xpdl xpdl) {
		super.importAttributes(xpp, xpdl);
		String value = xpp.getAttributeValue(null, "ParentLane");
		if (value != null) {
			parentLane = value;
		}
		value = xpp.getAttributeValue(null, "ParentPool");
		if (value != null) {
			parentPool = value;
		}
	}

	/**
	 * Exports all attributes.
	 */
	protected String exportAttributes() {
		String s = super.exportAttributes();
		if (parentLane != null) {
			s += exportAttribute("ParentLane", parentLane);
		}
		if (parentPool != null) {
			s += exportAttribute("ParentPool", parentPool);
		}
		return s;
	}

	protected void checkValidity(Xpdl xpdl) {
		super.checkValidity(xpdl);
	}
}
